from django.contrib import admin

from vto_core.models import TZWikiSlug
from vto_core.models import TZAbbreviation
from vto_core.models import TimeZone

# admin.site.register()

@admin.register(TZWikiSlug)
class TZWikiAdmin(admin.ModelAdmin):
    list_display = [ 'slug', ]
    search_fields = ['slug', ]

@admin.register(TZAbbreviation)
class TZAbbrAdmin(admin.ModelAdmin):
    autocomplete_fields = ['wiki_slug']
    list_display = [
        'offset',
        'abbr',
        'identifier',
        'wiki_slug',
    ]
    list_display_links = ['identifier']
    search_fields = [
        'identifier',
        'abbr',
    ]

@admin.register(TimeZone)
class TZAdmin(admin.ModelAdmin):
    autocomplete_fields = [
        'std',
        'dst',
    ]
    list_display = [
        'identifier',
        'tz_type',
        'src_file',
        'std',
        'dst',
        'aliases',
    ]
    search_fields = [
        'identifier',
        'std__identifier',
        'dst__identifier',
    ]


from datetime import datetime as dtm
from zoneinfo import ZoneInfo as zi
from django.conf import settings
from django.contrib import admin

from vto_core.models import DemoExchange, DemoTestTimestamp


@admin.register(DemoExchange)
class DemoExchangeAdmin(admin.ModelAdmin):
    autocomplete_fields = ['tz']
    list_display = [ 'abbr', 'name_long', 'tz' ]


@admin.register(DemoTestTimestamp)
class DemoTestTSAdmin(admin.ModelAdmin):
    list_display = [
        'ts_unix',
        'dst',

        'ts_utc',
        'get_utc',

        'get_utc_adm',

        'exchange',
        'exchange__tz',
    ]
    readonly_fields = [
        'ts_utc',
    ]


    #'''
    def get_utc_adm(self, obj):
        utca = dtm.fromtimestamp(obj.ts_unix, tz=zi('UTC'))
        return utca.strftime(settings.TSFMT)

    # get_utc_adm.admin_order_field = 'project__name'
    get_utc_adm.admin_order_field = 'ts_unix'
    # '''



from datetime import datetime as dtm
from zoneinfo import ZoneInfo as zi
from django.conf import settings
from django.views.generic import ListView 
from vto_core.models import DemoTestTimestamp


class DemoTestTSList(ListView):
    model = DemoTestTimestamp
    template_name = 'time/demo-ts-list.dtl'

    def get_context_data(self, **kwargs):

        context = super().get_context_data(**kwargs)

        for obj in context['object_list']:
            xch = dtm.fromtimestamp(obj.ts_unix, tz=zi(obj.exchange.tz.identifier))

            # in addition to the tz middleware converting the utc field in the model
            utc = dtm.fromtimestamp(obj.ts_unix, tz=zi('UTC'))

            # in addition to the tz template tag converting the user tz
            if self.request.user.is_authenticated:
                usr = dtm.fromtimestamp(obj.ts_unix, \
                    tz=zi(str(self.request.user.timezone.identifier)))

        context['ts_xch'] = xch.strftime(settings.TSFMT)
        
        # in addition to the tz middleware converting the utc field in the model
        context['tsv_utc'] = utc.strftime(settings.TSFMT)        

        # in addition to the tz template tag converting the user tz
        if self.request.user.is_authenticated:
            context['tsv_usr'] = usr.strftime(settings.TSFMT)

        return context

'''
            timestamp = obj.ts_unix
            timezone = zi(obj.exchange.tz)
            datetime_object = dtm.fromtimestamp(timestamp, tz=timezone)
            formatted_date = dt.strftime("%Y-%m-%d %H:%M:%S %z %Z")
        context['xch_ts] = formatted_date
'''


from datetime import datetime as dtm
from zoneinfo import ZoneInfo as zi
from django.db import models
from djinntoux.abstract_models import UUIDpk7


class DemoExchange(UUIDpk7):

    abbr = models.CharField(max_length=14, unique=True)

    name_long = models.CharField(max_length=30, unique=True)

    tz = models.ForeignKey('TimeZone', on_delete=models.PROTECT,
        null=True, blank=True)

    class Meta:
        ordering = ['name_long']

    def __str__(self):
        return self.abbr


''' samples
before DST: 1741035602
during DST: 
'''
class DemoTestTimestamp(models.Model):

    exchange = models.ForeignKey('DemoExchange', on_delete=models.PROTECT,
        to_field='abbr', default='NYSE')

    dst = models.BooleanField(default=False)

    ts_unix = models.PositiveIntegerField(help_text='1741035602')

    ts_utc = models.DateTimeField(null=True, blank=True)

    class Meta:
        ordering = ['ts_unix']
    
    def __str__(self):
        return str(self.ts_unix)
    
    def get_utc(self):
        """Convert Unix timestamp to UTC datetime"""
        if not self.ts_unix:
            return None
        
        try:
            # Create a timezone-aware UTC datetime from the Unix timestamp
            return dtm.fromtimestamp(self.ts_unix, tz=zi('UTC'))
        except (TypeError, ValueError) as e:
            print(f"Error converting to UTC: {e}")
            return None
    
    def save(self, *args, **kwargs):
        self.ts_utc = self.get_utc()
        super(DemoTestTimestamp, self).save(*args, **kwargs)



from django.db import models
from django.db.models import PositiveSmallIntegerField as PSInt
from djinntoux.abstract_models import UUIDpk7

'''
    more info:
https://en.wikipedia.org/wiki/List_of_tz_database_time_zones
'''

class TZWikiSlug(UUIDpk7):

    slug = models.CharField(max_length=50, unique=True,
        help_text='Western_European_Summer_Time')

    class Meta:
        ordering = ['slug']
        verbose_name = 'Tme zone Wikipedia slug'
        verbose_name_plural = 'Tme zone Wikipedia slugs'

    def __str__(self):
        return self.slug


class TZAbbreviation(UUIDpk7):

    identifier = models.CharField(max_length=30, unique=True,
        help_text='Central European Summer Time')

    offset = models.CharField(max_length=6, help_text='+02:00')

    abbr = models.CharField(max_length=4, unique=True, help_text='CEST')

    wiki_slug = models.ForeignKey('TZWikiSlug', on_delete=models.PROTECT,
        null=True, blank=True, related_name='rn_tz_abbr_wiki')

    class Meta:
        ordering = ['offset']
        verbose_name = 'Time zone abbreviation'
        verbose_name_plural = 'Tme zone abbreviations'

    def __str__(self):
        return self.abbr


class TimeZone(UUIDpk7):

    identifier = models.CharField(max_length=20, unique=True,
        help_text='America/New_York')

    class TZType(models.IntegerChoices):
        CAN = 1, 'canonical'
        LNK = 2, 'link (alias)'
        LKP = 3, 'link, previously canonical'

    tz_type = PSInt(choices=TZType.choices, default=TZType.CAN)

    class SrcFile(models.IntegerChoices):
        FAC = 1, 'factory'
        ETC = 2, 'etcetera'
        BAC = 3, 'backward'
        EUR = 4, 'europe'
        ASI = 5, 'asia'
        AUS = 6, 'australasia'
        NAM = 7, 'northamerica'
        SAM = 8, 'southamerica'
        AFR = 9, 'africa'
        ANT = 10, 'antarctica'

    src_file = PSInt(choices=SrcFile.choices, default=SrcFile.NAM)

    aliases = models.CharField(max_length=30, unique=True,
        help_text='US/Eastern, EST5EDT', null=True, blank=True)

    std = models.ForeignKey('TZAbbreviation', on_delete=models.PROTECT,
        related_name='rn_tz_abbr_std')

    # nullable, as some zones may not observe DST
    dst = models.ForeignKey('TZAbbreviation', on_delete=models.PROTECT,
        related_name='rn_tz_abbr_dst', null=True, blank=True)

    class Meta:
        ordering = ['identifier']

    def __str__(self):
        return self.identifier


from django.urls import path
from vto_core.views import TZList, DemoTestTSList

urlpatterns = [
    path('core/iana-tz-list/', TZList.as_view(), name='tz_list'),
    path('core/ts-render-demo/', DemoTestTSList.as_view(), name='ts_render_demo'),
]